#ifndef FIREWALL_H
#define FIREWALL_H

#include "Arduino.h"
#include "AsyncJson.h"
#include "ArduinoJson.h"
#include "EEPROM.h"
#ifdef ESP32
#include "WiFi.h"
#include "AsyncTCP.h"
#elif defined(ESP8266)
#include "ESP8266WiFi.h"
#include "ESPAsyncTCP.h"
#endif
#include "ESPAsyncWebServer.h"

typedef enum
{
    FW_REJECT = 0b0001,
    FW_DROP = 0b0010,
    FW_ACCEPT = 0b0011,
} firewall_target_t;

typedef enum
{
    FW_TCP = 0b0001,
    FW_UDP = 0b0010,
    FW_ALL = 0b1111,
} firewall_protocol_t;

typedef struct firewall_rule
{
    uint8_t key;
    char source[IP4ADDR_STRLEN_MAX];
    char destination[IP4ADDR_STRLEN_MAX];
    firewall_protocol_t protocol;
    firewall_target_t target;
    struct firewall_rule *next;
} firewall_rule_t;

class ESPFirewall
{
    uint8_t amount_of_rules = 0;
    int eeprom_address = 0;
    struct firewall_rule *head = NULL;

    AsyncWebServer *firewall_api;

    // Protocol / Target conversion
    String protocol_to_string(firewall_protocol_t &);
    firewall_protocol_t string_to_protocol(String &);
    String target_to_string(firewall_target_t &);
    firewall_target_t string_to_target(String &);

    // EEPROM
    void write_rule_to_eeprom(firewall_rule_t *rule);
    void get_rules_from_eeprom();

    // Firewall Actions
    void add_rule_to_firewall(firewall_rule_t *);
    firewall_rule_t *get_rule_from_firewall(int);
    bool delete_rule_from_firewall(int);

    // Firewall-API Actions
    void setup_routing();
    void json_message_response(AsyncWebServerRequest *, String, int);
    String construct_json_firewall_rule(firewall_rule_t *);
    String construct_json_firewall();
    void not_found(AsyncWebServerRequest *);
    void get_firewall_rule_handler(AsyncWebServerRequest *);
    void get_firewall_rules_handler(AsyncWebServerRequest *);
    bool request_has_firewall_parameter(AsyncWebServerRequest *);
    void post_firewall_handler(AsyncWebServerRequest *);
    void delete_firewall_handler(AsyncWebServerRequest *);

public:
    ESPFirewall(int port = 8080);
};

#endif
